.\" -*-nroff-*-
.\" ====================================================================
.\"  @Troff-man-file{
.\"     author          = "Nelson H. F. Beebe",
.\"     version         = "0.05",
.\"     date            = "13 May 2013",
.\"     time            = "06:33:13 MDT",
.\"     filename        = "bibtosql.man",
.\"     address         = "University of Utah
.\"                        Department of Mathematics, 110 LCB
.\"                        155 S 1400 E RM 233
.\"                        Salt Lake City, UT 84112-0090
.\"                        USA",
.\"     telephone       = "+1 801 581 5254",
.\"     FAX             = "+1 801 581 4148",
.\"     URL             = "http://www.math.utah.edu/~beebe",
.\"     checksum        = "58907 1941 7854 59577",
.\"     email           = "beebe@math.utah.edu, beebe@acm.org,
.\"                        beebe@computer.org (Internet)",
.\"     codetable       = "ISO/ASCII",
.\"     keywords        = "",
.\"     license         = "GNU General Public License (version 2 or
.\"                        later)",
.\"     supported       = "yes",
.\"     docstring       = "The checksum field above contains a CRC-16
.\"                        checksum as the first value, followed by the
.\"                        equivalent of the standard UNIX wc (word
.\"                        count) utility output of lines, words, and
.\"                        characters.  This is produced by Robert
.\"                        Solovay's checksum utility.",
.\"  }
.\" ====================================================================
.\"
.if t .ds Bi B\s-2IB\s+2T\\h'-0.1667m'\\v'0.20v'E\\v'-0.20v'\\h'-0.125m'X
.if n .ds Bi BibTeX
.\"
.if t .ds Te T\\h'-0.1667m'\\v'0.20v'E\\v'-0.20v'\\h'-0.125m'X
.if n .ds Te TeX
.\"
.\"=====================================================================
.TH BIBTOSQL 1 "13 May 2013" "Version 0.05"
.\"=====================================================================
.SH NAME
bibtosql \- convert BibTeX database files to SQL (Structured Query Language) database files
.\"=====================================================================
.SH SYNOPSIS
.B bibtosql
.\"-----------------------------------------------
.RB [ \-\|\-author ]
.\"-----------------------------------------------
.if n .ti +9n
.if t .ti +\w'\fBbibtosql\fP\ 'u
.RB [ \-\|\-create ]
.\"-----------------------------------------------
.if n .ti +9n
.if t .ti +\w'\fBbibtosql\fP\ 'u
.RB [ \-\|\-database
.IR dbname ]
.\"-----------------------------------------------
.if n .ti +9n
.if t .ti +\w'\fBbibtosql\fP\ 'u
.RB [ \-\|\-help ]
.\"-----------------------------------------------
.if n .ti +9n
.if t .ti +\w'\fBbibtosql\fP\ 'u
.RB [ \-\|\-server
.IR "[ MySQL | psql | PostgreSQL | SQLite ]" ]
.\"-----------------------------------------------
.if n .ti +9n
.if t .ti +\w'\fBbibtosql\fP\ 'u
.RB [ \-\|\-version ]
.\"-----------------------------------------------
.if n .ti +9n
.if t .ti +\w'\fBbibtosql\fP\ 'u
.IR "< infile" " or " " bibfile1 bibfile2 bibfile3 .\|.\|."
.\"-----------------------------------------------
.if n .ti +9n
.if t .ti +\w'\fBbibtosql\fP\ 'u
.IR "> outfile"
.\"=====================================================================
.SH DESCRIPTION
.B bibtosql
converts \*(Bi\& database files to a format
suitable for populating an SQL (Structured Query
Language) relational database where most of the
common bibliography entry keys
(\&\fCauthor\fP, \&\fCtitle\fP, \&\fCpublisher\fP,
\&\fCyear\fP, etc.)  are SQL table columns. The
original \*(Bi\& entries are also recorded in the
database tables as the \&\fCentry\fP column.
.PP
The acronym SQL is commonly pronounced either as
it is spelled
.RI ( "ess cue ell" ),
or like the name
.IR sequel .
.PP
SQL is defined in several national, government,
international, and industry standards, including
ANSI (X3.168-1989, X3.135-1992, 9579-2-1993,
9075-3-1995, and 9075-4-1996), FIPS (127:1990,
127-2:1993, 193:1995), ISO/IEC (9075:1987,
9075:1989, 9075:1992, 9075:2003, and 13249:2007),
and X/Open (CAE 1994).  Many 2008-vintage SQL
systems claim conformance to most of the 1992 ISO
Standard.
.PP
The data conversion provided by
.B bibtosql
gives access to the fast and powerful search and
display facilities of modern relational database
technology without giving up the convenience of
simple bibliographic text files that can be
prepared manually or with the help of other
bibliographic software.
.PP
In addition, since SQL databases are in wide use,
and most have powerful data-conversion facilities,
having bibliographic data accessible through an
SQL interface may facilitate its conversion to
other bibliographic formats, without the need to
handle \*(Bi\& files directly.
.PP
The companion
.BR bibsql (1)
utility provides a convenient interface to the SQL
database, hiding the unimportant details of how
the SQL client program is started, and where its
data are located in the filesystem or elsewhere on
the network.
.\"=====================================================================
.SH OPTIONS
Command-line options may be abbreviated to a
unique leading prefix, and lettercase is
.I not
significant.
.PP
The leading hyphen that distinguishes an option
from a filename may be doubled, for compatibility
with GNU and POSIX conventions.  Thus,
.BR \-a ,
.BR \-author
and
.B \-\|\-author
are equivalent.
.PP
.\"-----------------------------------------------
.TP \w'\-database'u+3n
.B \-\|\-author
Display an author credit on
.IR stdout ,
and then terminate with a success return code.
.\"-----------------------------------------------
.TP
.B \-\|\-create
Output leading SQL commands to delete any existing
data, and create a new \*(Bi\& table for key/value
searching.  Without this option, output consists
only of commands for adding new records to an
existing database.
.\"-----------------------------------------------
.TP
.BI \-\|\-database " name of \*(Bi\& database"
Specify the name to be used for the database in
the SQL system.  The default name is
.IR bibtex .
.IP
This option exists primarily for experimentation,
and should rarely be needed.
.\"-----------------------------------------------
.TP
.B \-\|\-help
Display a help message on
.IR stdout ,
giving a usage description, similar to this
section of the manual pages, and then terminate
with a success return code.
.\"-----------------------------------------------
.TP
.BI \-\|\-server " [ MySQL | psql | PostgreSQL | SQLite ]"
Select the database server type for which input is to
be prepared (default:
.IR SQLite ).
.IP
The name
.I psql
is an alias for
.IR PostgreSQL ,
since the former is the name of the client program
on many systems.
.IP
The name may be abbreviated to any unique leading
prefix, and its lettercase is
.I not
significant.
.IP
Although the input formats of the supported SQL
systems are similar, there are important small
differences that make it imperative to identify
the target system.
.\"-----------------------------------------------
.TP
.B \-\|\-version
Display the program version number and release
date on
.I stdout
and then terminate with a success return code.
.\"-----------------------------------------------
.TP
.B \-\|\-
Terminate options.  All remaining arguments are
treated as filenames.
.\"=====================================================================
.SH "DETAILED DESCRIPTION"
.PP
In order to keep the data-conversion task as simple as
possible,
.B bibtosql
assumes that the \*(Bi\& input files have been
standardized, either manually, or more easily,
with the help of
.BR bibclean (1).
In particular,
.B bibtosql
assumes the following conventions:
.\"-----------------------------------------------
.TP \w'\(bu'u+2n
\(bu
Comment lines begin with a percent in the first
column, and can be discarded for the purposes of
the database conversion.
.\"-----------------------------------------------
.TP
\(bu
\*(Bi\& entries begin with an @ character in column 1,
immediately followed by a document type, an open brace, a
citation label, and a comma.
.\"-----------------------------------------------
.TP
\(bu
Key/value pairs start on separate lines, and the
value is a quoted string followed by a comma.  The
value string may span multiple lines, but it must
start on the same line as the key.  The quoted
string may be replaced by a single name of a
\*(Bi\& \&\fC@String{.\|.\|.}\fP definition.
.\"-----------------------------------------------
.TP
\(bu
\*(Bi\& entries end with a single close brace at
the beginning of a line.
.PP
Any \*(Bi\& line may optionally have trailing whitespace,
which is ignored.
.PP
These conventions are most easily illustrated by
two simple examples, one fictional, and one real:
.RS
.nf
\&\fC@Book{Jones:2008:GGE,
  author =       "Terry Jones",
  title =        "Gnat and Gnu Entozoology",
  publisher =    "The National Zoo Press",
  address =      "Washington, DC, USA",
  pages =        "xx + 723",
  year =         "2008",
  ISBN =         "0-9999999-9-0",
  ISBN-13 =      "978-0-9999999-9-8",
}

@Article{Boyer:2002:MEW,
  author =       "R. S. Boyer and W. Feijen and D. Gries and
                 C. A. R. Hoare and J. Misra and J. Moore
                 and H. Richards",
  title =        "In memoriam: {Edsger W. Dijkstra}
                 1930--2002",
  journal =      j-CACM,
  volume =       "45",
  number =       "10",
  pages =        "21--22",
  month =        oct,
  year =         "2002",
  CODEN =        "CACMA2",
  ISSN =         "0001-0782",
  bibdate =      "Wed Sep 3 17:06:29 MDT 2003",
  bibsource =    "http://www.acm.org/pubs/contents/journals/cacm/",
  acknowledgement = ack-nhfb,
}\fP
.fi
.RE
.PP
\*(Bi\& can process a compressed entry of the form
.RS
.nf
\&\fC@Book{Jones:2008:GGE,author={Terry Jones},title=
  {Gnat and Gnu Entozoology},publisher={The Zoo Press},
  address={Washington,DC,USA},pages={xx + 723},
  year={2008},ISBN={0-9999999-9-0},ISBN-13={978-0-9999999-9-8}}\fP
.fi
.RE
but
.B bibtosql
and most other simple \*(Bi\& tools cannot.
.PP
The output of
.B bibtosql
consists of optional SQL commands to create a
database, followed by a transaction block that
contains zero or more SQL commands to insert data
into the database in either of two tables, one for
document entries, and the other for \*(Bi\&
\&\fC@String{key = value}\fP definitions.  The
data conversion provided by
.B bibtosql
is much faster than the loading of the converted
data into the SQL database system, but each task
needs to be done only once, as long as the
database input format is not altered.  Subsequent
changes to the \*(Bi\& data can be converted by
.B bibtosql
and entered into an SQL database previously
created with input from that tool, without having
to recreate the entire database.
.PP
Once the SQL database is available, starting the
SQL program and issuing queries is relatively
quick, and search responses can be produced in a
fraction of a second on modern personal and office
computers, even for a database with a million or
so entries.
.PP
The database creation is initiated by an input
block that might look something like this:
.RS
.nf
\&\fCCREATE TABLE bibtab (
        authorcount  INTEGER,
        editorcount  INTEGER,
        pagecount    INTEGER,
        bibtype      TEXT,
        filename     TEXT,
        label        TEXT,
        author       TEXT,
        editor       TEXT,
        booktitle    TEXT,
        title        TEXT,
        crossref     TEXT,
        chapter      TEXT,
        journal      TEXT,
        volume       TEXT,
        type         TEXT,
        number       TEXT,
        institution  TEXT,
        organization TEXT,
        publisher    TEXT,
        school       TEXT,
        address      TEXT,
        edition      TEXT,
        pages        TEXT,
        day          TEXT,
        month        TEXT,
        monthnumber  TEXT,
        year         TEXT,
        CODEN        TEXT,
        DOI          TEXT,
        ISBN         TEXT,
        ISBN13       TEXT,
        ISSN         TEXT,
        LCCN         TEXT,
        MRclass      TEXT,
        MRnumber     TEXT,
        MRreviewer   TEXT,
        bibdate      TEXT,
        bibsource    TEXT,
        bibtimestamp TEXT,
        note         TEXT,
        series       TEXT,
        URL          TEXT,
        abstract     TEXT,
        fjournal     TEXT,
        keywords     TEXT,
        language     TEXT,
        remark       TEXT,
        subject      TEXT,
        TOC          TEXT,
        ZMnumber     TEXT,
        acknowledgement TEXT,
        advisor      TEXT,
        affiliation  TEXT,
        affiliationaddress TEXT,
        ajournal     TEXT,
        annote       TEXT,
        articleno    TEXT,
        authordates  TEXT,
        bookDOI      TEXT,
        bookURL      TEXT,
        bookpages    TEXT,
        classcodes   TEXT,
        corpsource   TEXT,
        editordates  TEXT,
        howpublished TEXT,
        journalURL   TEXT,
        journalabr   TEXT,
        key          TEXT,
        onlinedate   TEXT,
        reviewer     TEXT,
        subjectdates TEXT,
        thesaurus    TEXT,
        treatment    TEXT,
        ZMclass      TEXT,
        entry        TEXT NOT NULL UNIQUE
);

CREATE TABLE namtab (
        name         TEXT NOT NULL UNIQUE,
        count        INTEGER
);

CREATE TABLE strtab (
        key          TEXT,
        value        TEXT,
        entry        TEXT NOT NULL UNIQUE
);\fP
.fi
.RE
.PP
The key values are mostly character strings, and
the key names are available for use in subsequent
searches in the database.  The lettercase of key
names is not significant.  The values are of type
\&\fCTEXT\fP, a character string type that holds
at least 65,535 characters on all supported SQL
databases.  As described later, MySQL requires a
following parenthesized length field.
.PP
The key names \&\fCauthor\fP through
\&\fCZMnumber\fP are given in the \&\fCCREATE\fP
command in the order chosen by
.BR biborder (1),
which is similar to that of the field order in
conventional publication and reference lists.
That order has no significance for either
searching or database efficiency.
.PP
While most of the key names in the \&\fCbibtab\fP
table are standard ones in \*(Bi\&, a few are
not. They include
.\"-----------------------------------------------
.TP \w'\(bu'u+2n
\(bu
\&\fCCODEN\fP (Chemical Abstracts
.I "CODE Name"
for serial publications).
.\"-----------------------------------------------
.TP
\(bu
\&\fCDOI\fP
.RI ( "Digital Object Identifier",
a
.I "unique and permanent"
name for the master copy of an electronic
document).  If it does not look like an Internet
URL, convert it to one by prefixing it with
\&\fChttp://dx.doi.org/\fP.
.\"-----------------------------------------------
.TP
\(bu
\&\fCISSN\fP
.RI ( "International Standard Serial Number" :
eight digits, optionally written as two groups of
four digits separated by a hyphen).
.\"-----------------------------------------------
.TP
\(bu
\&\fCISBN\fP (old-style
.IR "International Standard Book Number" :
10 digits, the last of which may be
.I X
or
.IR x ).
Optional hyphens separate it into four digit
groups that define the country or language,
publisher, book number, and a final check digit
for error detection.
.\"-----------------------------------------------
.TP
\(bu
\&\fCISBN13\fP (new-style (since 2007)
.IR "International Standard Book Number" :
13 digits, the last of which may be
.I X
or
.IR x ).
This contains the old ten-digit ISBN, with a new
prefix of \&\fC978-\fP, and a revised check digit.
When the supply of unused ISBNs is exhausted, a
new prefix \&\fC979-\fP will be assigned, and such
numbers will then not be convertible to the
old ten-digit form.
.\"-----------------------------------------------
.TP
\(bu
\&\fCLCCN\fP (US
.IR "Library of Congress Catalog Number" ),
used for book identification at many US libraries,
and at some libraries in other countries.
.\"-----------------------------------------------
.TP
\(bu
\&\fCMRclass\fP (list of American Mathematical
Society
.I "Math Reviews"
five-character subject classification codes).
.\"-----------------------------------------------
.TP
\(bu
\&\fCMRnumber\fP
.RI ( "Math Reviews" " database number)."
.\"-----------------------------------------------
.TP
\(bu
\&\fCajournal\fP (abbreviated journal name).
.\"-----------------------------------------------
.TP
\(bu
\&\fCarticleno\fP (article number).
.\"-----------------------------------------------
.TP
\(bu
\&\fCbibdate\fP (Unix
.BR date (1)
string in the formats \&\fCSat Oct 25 20:15:00 MDT
2008\fP or \&\fCSat Oct 25 20:15:00 2008\fP; it
records a timestamp of the last significant change to
the \*(Bi\& entry data).
.\"-----------------------------------------------
.TP
\(bu
\&\fCbibsource\fP (arbitrary text indicating
source(s) of bibliographic data, often as a list of
Internet URLs or Z3950 library catalog addresses).
.\"-----------------------------------------------
.TP
\(bu
\&\fCbibtimestamp\fP (alternate representation of
\&\fCbibdate\fP value in the form \&\fC2008.10.25
20:15:00 MDT\fP, for which lexicographic sort
order is also time order).
.\"-----------------------------------------------
.TP
\(bu
\&\fCURL\fP (Internet
.I "Uniform Resource Locator(s)"
for retrieval of an electronic form of the
document).
.\"-----------------------------------------------
.TP
\(bu
\&\fCremark\fP (additional commentary about the
entry that is not intended to be included in a
typeset reference list).
.\"-----------------------------------------------
.TP
\(bu
\&\fCsubject\fP (subject classification phrases).
.\"-----------------------------------------------
.TP
\(bu
\&\fCTOC\fP (table of contents of the publication).
.\"-----------------------------------------------
.TP
\(bu
\&\fCZMclass\fP (European Mathematical Society
.I "Zeitschrift fuer Mathematik"
subject classification codes).
.\"-----------------------------------------------
.TP
\(bu
\&\fCZMnumber\fP (European Mathematical Society
.I "Zeitschrift fuer Mathematik"
database number).
.PP
In
.BR bibtosql ,
we follow the common SQL practice that
indeterminate, unknown, or unset values are output
as NULL, so as to facilitate their exclusion in
later searches.
.PP
The \&\fCNOT NULL\fP attribute on the
\&\fCentry\fP key requires that column to be
supplied in each row: it is the only mandatory row
entry.
.PP
The \&\fCUNIQUE\fP attribute on the \&\fCentry\fP
key ensures that only one distinct copy of each
\*(Bi\& entry is stored in the database, even
when identical copies are present in the input
stream.  That situation is common, since \*(Bi\&
entries are often copied between files and shared
between users.
.PP
For PostgreSQL and SQLite, the \&\fCTEXT\fP type
can represent a string of any size.  For MySQL,
that type must be followed by a parenthesized
typical length: we use (32767).  If that length is
too small, long strings are arbitrarily, and
silently, truncated.
.PP
The data-insertion block looks like this:
.RS
.nf
\&\fCBEGIN TRANSACTION;
\&.\|.\|. INSERT \fRcommands go here\fP .\|.\|.
COMMIT;\fP
.fi
.RE
.PP
This allows the database system to delay
committing the input to permanent database storage
until all of the input has been processed
successfully.  Any error since the start of the
transaction causes the entire batch of updates to
be discarded.
.PP
A single insertion command might look like this:
.RS
.nf
\&\fCINSERT INTO bibtab
        (authorcount, editorcount, pagecount, bibtype, filename, label,
        author, editor, booktitle, title, crossref, chapter, journal,
        volume, type, number, institution, organization, publisher,
        school, address, edition, pages, day, month, monthnumber, year,
        CODEN, DOI, ISBN, ISBN13, ISSN, LCCN, MRclass, MRnumber,
        MRreviewer, bibdate, bibsource, bibtimestamp, note, series, URL,
        abstract, fjournal, keywords, language, remark, subject, TOC,
        ZMnumber, entry)
        VALUES (
        NULL,
        5,
        549,
        'book',
        'master.bib',
        'Goossens:2008:LGC',
        NULL,
        'Michel Goossens and Frank Mittelbach and Sebastian Rahtz and \e
Denis Roegel and Herbert Voss',
        NULL,
        'The LaTeX Graphics Companion',
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        'pub-AW',
        NULL,
        'pub-AW:adr',
        'Second',
        'xxi + 528',
        NULL,
        NULL,
        NULL,
        '2008',
        NULL,
        NULL,
        '0-321-50892-0 OR 0321508920',
        '978-0-321-50892-8 OR 9780321508928',
        NULL,
        'Z253.4.L38G663 2008',
        NULL,
        NULL,
        NULL,
        'Thu Sep 20 12:02:17 MDT 2007',
        'z3950.bibsys.no:2100/BIBSYS',
        '2007.09.20 12:02:17 MDT',
        NULL,
        'Addison-Wesley series on tools and techniques for computer \e
typesetting',
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        NULL,
        '
@Book{Goossens:2008:LGC,
  editor =       "Michel Goossens and Frank Mittelbach and Sebastian
                 Rahtz and Denis Roegel and Herbert Vo{\ess}",
  title =        "The {\eLaTeX} Graphics Companion",
  publisher =    pub-AW,
  address =      pub-AW:adr,
  edition =      "Second",
  year =         "2008",
  ISBN =         "0-321-50892-0",
  ISBN-13 =      "978-0-321-50892-8",
  LCCN =         "Z253.4.L38G663 2008",
  bibdate =      "Thu Sep 20 12:02:17 MDT 2007",
  bibsource =    "z3950.bibsys.no:2100/BIBSYS",
  series =       "Addison-Wesley series on tools and techniques for
                 computer typesetting",
  acknowledgement = ack-nhfb,
}'
);\fP
.fi
.RE
.PP
Long lines have been wrapped with a
backslash-newline for display purposes here; such
wrapping is absent from the program output.
.PP
The \&\fCISBN\fP and \&\fCISBN13\fP values are
augmented with hyphen-free book numbers.  Although
the hyphen positions are well defined, they are
usually unpredictable to humans, and may be
incorrect in hand-produced bibliographic data.
The hyphen-free form provides another way to find
them in searches.
.PP
The \&\fCINSERT\fP statement is clearly verbose,
and somewhat awkward and error-prone because of
the separation of keys from their values, but that
matters little for a computer program.  When all
fields are provided in the expected order, the
parenthesized list of key names can be omitted.
However,
.B bibtosql
intentionally includes the key names to ensure
compatibility of its output file with databases
created by future versions of the program that
might define additional columns.
.PP
Manual insertions later can provide data for
subsets of keys; any that are omitted simply
acquire NULL values.
.PP
.I Except
in the value of the \&\fCentry\fP key, consecutive
whitespace is reduced to a single space, \*(Te\&
macros are simplified or removed, and \*(Te\& accents
and braces are stripped from key values.  That
convention makes specification of search strings
for key values easier.  Thus, to search for
\&\fCPaul Erd{\eH{o}}s\fP, use the pattern
\&\fC%P%Erdos%\fP in the \&\fCauthor\fP or
\&\fCeditor\fP columns.  To find documents with
\&\fC{\eTeX}book\fP or \&\fC\eTeX{}book\fP or
\&\fC\eTeX book\fP in their titles, use the
pattern \&\fC%texbook%\fP.  Search patterns are
described in the
.B "SQL SEARCH TUTORIAL"
section of the manual pages for
.BR bibsql (1).
.\"=====================================================================
.SH "SQLite SERVER SETUP"
SQLite is the simplest of the databases supported
by
.B bibtosql
and
.BR bibsql (1).
The same program,
.BR sqlite3 (1),
acts as both a client and a server, and thus, the
database must reside on the same system as the
client.  There is no data protection other than
that provided by the host filesystem.  In contrast
to the complex access controls of client/server
database systems, with SQLite, no special user
privileges are needed to create or use an SQLite
database.
.PP
SQLite is an excellent system for users new to
databases to learn about, and experiment with,
SQL.
.PP
The SQLite software is
.I "public domain"
and may be used for any purpose, and
redistributed, without restriction.  The software
is fast, portable, and reliable.  It can be built
easily on almost any modern desktop operating
system, including all current Unix, Mac OS X, and
Windows systems.  Package downloads and
documentation are available at the developer site,
\&\fChttp://www.sqlite.org/\fP.
.PP
An interesting feature of SQLite is that there is
an amalgamated distribution where the many
ordinarily-separate source code files are
collected into just two large files, enabling
advanced compilers to do interprocedural
optimizations and procedure inlining, further
enhancing database performance.
.PP
An SQLite database consists of a
.I "single file"
that is independent of operating system, CPU
architecture, and storage byte order.  Once
created, an SQLite database file can be used on
any system where SQLite can be compiled, and is
ideally suited for distribution on read-only media
such as CD-ROMs or DVDs, or via the Web.
.PP
To create an SQLite database, name it on the
command line, and then input a file of SQL
commands to populate the database:
.RS
.nf
.B "% bibtosql *.bib | sqlite3 bibtex.db"
.fi
.RE
To search the database, just run the program again
with the same filename argument:
.RS
.nf
.B "% sqlite3 bibtex.db"
\&\fCsqlite> \fP\&\fB.\|.\|. your input here .\|.\|.\fP
.fi
.RE
If a database filename is not provided, then
SQLite creates an in-memory database that is lost
when the program terminates.  This is useful for
testing new data and new table formats, as well as
for fast searching of small private collections of
\*(Bi\& data.  Here is a sample in-memory session:
.RS
.nf
.B "% sqlite3"
\&\fCsqlite> \&\fB.read testdata.sql\fP\fP
\&\fCsqlite> \&\fB.\|.\|. your input here .\|.\|.\fP\fP
.fi
.RE
.PP
For use with
.BR bibsql (1),
the SQLite database file should be created and
copied to the installation directory, and then set
read-only to prevent unintentional or malicious
modification.  Assuming the default prefix of
\&\fC/usr/local\fP, the job can be done like this:
.RS
.nf
.B "% cp bibtex.db /usr/local/share/lib/bibsql/bibsql-x.yz/"
.B "% chmod a-wx,a+r /usr/local/share/lib/bibsql/bibsql-x.yz/bibtex.db"
.fi
.RE
.PP
If the database file is large, searches may be
made faster by creating indexes of important parts
of the data:
.RS
.nf
\&\fCsqlite> \fP\&\fBcreate index bibidx on bibtab (author, title, label);\fP
\&\fCsqlite> \fP\&\fBcreate index namidx on namtab (name);\fP
\&\fCsqlite> \fP\&\fBcreate index stridx on strtab (key, value);\fP
.fi
.RE
.PP
.\"=====================================================================
.SH "MySQL SERVER SETUP"
Setting up the server side of a MySQL connection
is complex, and these step-by-step notes may be a
helpful guide.  It is a good thing to log what you
are doing, either with the log facility provided
in an
.BR xterm (1)
window (usually on a menu bound to Ctl-Button-1),
or else with the
.BR script (1)
command.
.PP
We assume that the MySQL software has been
installed in the directory \&\fC/usr/local/bin\fP,
and its databases are to be created in
subdirectories under the directory
\&\fC/var/lib/mysql\fP, named by the database.
For clarity, we use three different prompt
strings: \&\fC#\fP for the \&\fCroot\fP user,
\&\fC$\fP for the MySQL superuser, and \&\fC%\fP
for an unprivileged ordinary user.
.\"-----------------------------------------------
.TP \w'\(bu'u+2n
\(bu
If you have not yet done so, create a Unix account
for the MySQL superuser:
.RS
.nf
.B "# adduser mysql"
.fi
.RE
.IP
This account has no special Unix privileges, and
exists only to restrict access to database files,
and to provide an account under which the MySQL
server daemon runs.
.IP
In addition, it has no password set, so direct
login with this account is impossible.  Instead,
you must first become the \&\fCroot\fP user, and
then use the
.BR su (1)
or
.BR sudo (1)
commands to change to the \&\fCmysql\fP account.
.\"-----------------------------------------------
.TP
\(bu
Create a working directory and a log file owned by
the \&\fCmysql\fP account:
.RS
.nf
.B "# mkdir /var/run/mysqld/"
.B "# chown mysql.mysql /var/run/mysqld"
.B "# touch /var/log/mysqld.log"
.B "# chown mysql.mysql /var/log/mysqld.log"
.fi
.RE
.IP
Consult the log file to help diagnose database
errors and connection problems.
.\"-----------------------------------------------
.TP
\(bu
Become the MySQL superuser and start the MySQL
server:
.RS
.nf
.B "# su \(mi mysql"
.B "$ cd /usr/local/mysql"
.B "$ mysqld_safe &"
\&\fCnohup: ignoring input and redirecting stderr to stdout
Starting mysqld daemon with databases from /var/lib/mysql\fP
.fi
.RE
.IP
This program is a wrapper around the actual
daemon,
.BR mysql (1),
that handles incoming SQL connections.  The
wrapper monitors the daemon process, and
automatically restarts it, should it fail.  We
show later how to get it started each time the
server system reboots.
.IP
As long as there were no errors, the MySQL daemon
is almost ready to handle both local and network
connections from MySQL clients, as soon as we
create a database and MySQL user names.
.\"-----------------------------------------------
.TP
\(bu
Check the running processes to find where various
things are located:
.RS
.nf
.B "$ ps auxww | grep \(mii sql"
\&\fC.\|.\|.
mysql 11795 0.0 0.0  52776 1132  pts/6 S  21:58 0:00 \e
    /bin/sh /usr/local/bin/mysqld_safe
mysql 11821 0.2 0.5 131092 17032 pts/6 Sl 21:58 0:00 \e
    /usr/local/libexec/mysqld \e
    \(mi\|\(mibasedir=/usr/local \e
    \(mi\|\(midatadir=/var/lib/mysql \e
    \(mi\|\(mipid-file=/var/run/mysqld/mysqld.pid \e
    \(mi\|\(miskip-external-locking \e
    \(mi\|\(misocket=/var/lib/mysql/mysql.sock
\&.\|.\|.\fP
.fi
.RE
.IP
We wrapped the long process reports with
backslash-newline to make them fit here, and also
to highlight the options passed by the wrapper to
the daemon.  Notice, in particular, the socket
setting, because we need it later.  A Unix
.I socket
is the local equivalent of an external network
connection: the socket allows interprocess
communication on the same machine.  When we start
a MySQL client later, we have to tell it what
socket to use to reach the MySQL server.
.IP
If you record the socket pathname in an
environment variable, like this,
.RS
.nf
.B "$ MYSQL_UNIX_PORT=/var/lib/mysql/mysql.sock"
.B "$ export MYSQL_UNIX_PORT"
.fi
.RE
.IP
then it need not be supplied in later commands.
.\"-----------------------------------------------
.TP
\(bu
Check that the server is listening for client connections:
.RS
.nf
.B "% netstat \(mian | egrep \(mii '1186|2273|3306|mysql'"
\&\fC
tcp  0 0 0.0.0.0:3306        0.0.0.0:*            LISTEN
tcp  0 0 155.101.96.167:3306 155.101.96.19:47297  ESTABLISHED
tcp  0 0 155.101.96.167:3306 155.101.96.132:27429 ESTABLISHED
unix 2 [ ACC ] STREAM LISTENING 3885498 /var/lib/mysql/mysql.sock
.fi
.RE
.IP
That command is another way to find out the socket
path.  The numbers in the search pattern are
standard network ports reserved for MySQL and
recorded in the file \&\fC/etc/services\fP.
.\"-----------------------------------------------
.TP
\(bu
Initialize the database directory tree with
template files:
.RS
.nf
.B "$ mysql_install_db \(mi\|\(miuser=mysql"
.fi
.RE
.IP
That command produces a lengthy report that you
should record.  It gives instructions for setting
up MySQL user accounts and creating databases, and
points you to free online documentation and
commercial support.
.\"-----------------------------------------------
.TP
\(bu
Create three MySQL user names (unrelated to Unix
accounts) to be assigned access privileges
shortly:
.RS
.nf
.B "$ mysqladmin \(mi\|\(misocket=/var/lib/mysql/mysql.sock \(miu root password '/s/e/c/r/e/t/'"
.B "$ mysqladmin \(mi\|\(misocket=/var/lib/mysql/mysql.sock \(miu bibtex password '/t/e/r/c/e/s/'"
.B "$ mysqladmin \(mi\|\(misocket=/var/lib/mysql/mysql.sock \(miu reader"
.fi
.RE
.IP
You should, of course, use different, and
harder-to-guess, passwords than shown here.
.IP
If \&\fCMYSQL_UNIX_PORT\fP is not set, then we
have to supply the socket location.  Without it,
MySQL programs assume that the socket resides in
the \&\fC/tmp\fP directory, and quit when they
fail to find it.
.IP
.B "WARNING WARNING WARNING:"
There is a clear security hole here: anyone else
on the system who is running a
.BR ps (1)
command with options for verbose output can see
the entire command lines, including the passwords.
For that, and other security reasons, database
server machines should have extremely restricted
access, with only system-manager login accounts.
.IP
When you run a MySQL client with the
password-protected user names, the client demands
\&\fCEnter password:\fP before proceeding.  The
password that you supply at that point is program
input, and is neither echoed as you type, nor
visible in
.BR ps (1)
command output.
.IP
The chapter
.I "MySQL Server Administration"
in the
.I "MySQL Reference Manual"
contains sections entitled
.I "Assigning Account Passwords"
and
.I "Keeping Your Password Secure"
that describe other, possibly more secure, ways to
supply passwords.
.\"-----------------------------------------------
.TP
\(bu
Create an empty database for the bibliographic
data:
.RS
.nf
.B "$ mysqladmin \(miu root \(mip create bibtex \(mi\|\(misocket=/var/lib/mysql/mysql.sock"
\&\fCEnter password: \&\fB/s/e/c/r/e/t/\fP\fP
.fi
.RE
.IP
The \&\fB\(mip\fP option requests a password
prompt, as shown.  The alternative is to supply it
on the command line as the option
\&\fB\(mi\|\(mipassword='/s/e/c/r/e/t/'\fP, but that
is insecure and inadvisable. It may, however, be
necessary if the command is run in batch mode
where a prompt is not possible.
.IP
There is no screen output from this command, but
it creates a directory
\&\fC/var/lib/mysql/bibtex\fP containing a single
file, \&\fCdb.opt\fP, that specifies the default
character set and sorting order.
.\"-----------------------------------------------
.TP
\(bu
If the database directory is on a filesystem that
is accessible via network mounts to clients, for
security and performance reasons, consider moving
it now to another filesystem that is strictly
local.  If you make such a move, you must then
leave behind a symbolic link:
.RS
.nf
.B "$ mv /var/lib/mysql/bibtex /local/mysql/"
.B "$ ln \(mis /local/mysql/bibtex /var/lib/mysql/"
.fi
.RE
.\"-----------------------------------------------
.TP
\(bu
Assign database-access privileges to the newly-created accounts:
.RS
.nf
.B "$ mysql \(mip \(mi\|\(misocket=/var/lib/mysql/mysql.sock \(miu root"
\&\fCEnter password: \&\fB/s/e/c/r/e/t/\fP\fP
\&.\|.\|.
\&\fCmysql> \fP\&\fBGRANT ALL PRIVILEGES ON bibtex.* TO 'bibtex'@'localhost';\fP
\&\fCQuery OK, 0 rows affected (0.00 sec)\fP
.B ""
\&\fCmysql> \fP\&\fBGRANT SELECT ON bibtex.* TO 'reader'@'%';\fP
\&\fCQuery OK, 0 rows affected (0.00 sec)\fP
.fi
.RE
.IP
Now the user \&\fCbibtex\fP can create and
populate tables in the \&\fCbibtex\fP database,
but only from the local host where the server is
running.  For security reasons, it is not
advisable to allow changes to the database from
hosts other than the MySQL server system.
.IP
The user \&\fCreader\fP can read tables in the
\&\fCbibtex\fP database from any host on the
Internet.  Further restrictions may be advisable.
If so, instead of a percent meaning all host
names, remove all access from that user, then
limit access to hosts in your domain:
.RS
.nf
.B "\&\fCmysql> \&\fBREVOKE ALL PRIVILEGES, GRANT OPTION FROM 'reader'@'%';\fP\fP"
.B "\&\fCQuery OK, 0 rows affected (0.00 sec)\fP"
.B ""
.B "\&\fCmysql> \&\fBGRANT SELECT ON bibtex.* TO 'reader'@'%.example.com';\fP\fP"
.B "\&\fCQuery OK, 0 rows affected (0.00 sec)\fP"
.fi
.RE
.IP
You can also use a numeric IP address to permit a
connection from a specific host:
.RS
.nf
.B "\&\fCmysql> \&\fBgrant select on bibtex.* to 'reader'@'192.168.1.103';\fP"
.fi
.RE
.\"-----------------------------------------------
.TP
\(bu
Find the accounts in database tables and check
their access privileges:
.RS
.nf
.B "\&\fCmysql> \fPshow databases;"
\&\fC+--------------------+
| Database           |
+--------------------+
| information_schema |
| bibtex             |
| mysql              |
| test               |
+--------------------+\fP
.B ""
.B "\&\fCmysql> \fPuse information_schema;"
.B ""
.B "\&\fCmysql> \fPshow tables;"
\&\fC+---------------------------------------+
| Tables_in_information_schema          |
+---------------------------------------+
| CHARACTER_SETS                        |
| COLLATIONS                            |
\&.\|.\|.
| USER_PRIVILEGES                       |
| VIEWS                                 |
+---------------------------------------+
.B "\&\fCmysql> \fPselect * from USER_PRIVILEGES;"
\&\fC+--------------------------+---------------+----------------+--------------+
| GRANTEE                  | TABLE_CATALOG | PRIVILEGE_TYPE | IS_GRANTABLE |
+--------------------------+---------------+----------------+--------------+
| 'root'@'localhost'       | NULL          | SELECT         | YES          |
\&...
| 'root'@'127.0.0.1'       | NULL          | CREATE USER    | YES          |
| 'bibtex'@'localhost'     | NULL          | USAGE          | NO           |
| 'reader'@'%.example.com' | NULL          | USAGE          | NO           |
+--------------------------+---------------+----------------+--------------+\fP
.fi
.RE
.\"-----------------------------------------------
.TP
\(bu
Now populate the database with suitable data
produced by
.BR bibtosql :
.RS
.nf
.B "$ mysql \(mip \(miu bibtex \(miD bibtex \(mi\|\(misocket=/var/lib/mysql/mysql.sock \e"
.B "    < \&\fI/path/to/data/\fPtest-mysql.sql \e"
.B "    > logs/bibtex-`hostname`.log 2>&1"
\&\fCEnter password: \&\fB/s/e/c/r/e/t/\fP\fP
.fi
.RE
.IP
This step can take a long time if the input file
is large.
.IP
After a test with a file of about 11\|400 \*(Bi\&
entries, the database directory looks something
like this:
.RS
.nf
.B "$ ls \(milFd /var/lib/mysql/bibtex"
\&\fCdrwx------  2 mysql mysql 4096 2008-11-01 17:31 /var/lib/mysql/bibtex/\fP
.B ""
.B "$ ls \(mil /var/lib/mysql/bibtex"
\&\fCtotal 16104
-rw-rw---- 1 mysql mysql    10218 2008-11-01 23:49 bibtab.frm
-rw-rw---- 1 mysql mysql 15976280 2008-11-01 23:49 bibtab.MYD
-rw-rw---- 1 mysql mysql     1024 2008-11-01 23:49 bibtab.MYI
-rw-rw---- 1 mysql mysql       65 2008-11-01 23:49 db.opt
-rw-rw---- 1 mysql mysql     8592 2008-11-01 23:49 namtab.frm
-rw-rw---- 1 mysql mysql   299268 2008-11-01 23:49 namtab.MYD
-rw-rw---- 1 mysql mysql     1024 2008-11-01 23:49 namtab.MYI
-rw-rw---- 1 mysql mysql     8622 2008-11-01 23:49 strtab.frm
-rw-rw---- 1 mysql mysql   124408 2008-11-01 23:49 strtab.MYD
-rw-rw---- 1 mysql mysql     1024 2008-11-01 23:49 strtab.MYI\&\fC
.B ""
.B "$ du \(mih /var/lib/mysql/bibtex"
\&\fC16M     /var/lib/mysql/bibtex\fP
.fi
.RE
.IP
Notice that the directory and file permissions
allow access only by the \&\fCmysql\fP user.
.IP
A simple calculation shows that the database
consumes about 1400 bytes of filesystem space for
an average \*(Bi\& entry.  We can compare this
with a report from MySQL itself:
.RS
.nf
.B "\&\fCmysql> \&\fBselect avg(length(entry)) from bibtab;\fP\fP"
\&\fC+--------------------+
| avg(length(entry)) |
+--------------------+
|           860.1307 |
+--------------------+
1 row in set (0.32 sec)\fP
.fi
.RE
.IP
The count of 860 bytes represents just the
\&\fCentry\fP string, and does not include the
space needed for the other four dozen or so
columns in each row of the \&\fCbibtex\fP
database.
.\"-----------------------------------------------
.TP
\(bu
You can get lengthy reports of MySQL default variable
settings, and a list of all variables, like this:
.RS
.nf
.B "$ mysqladmin \(mi\|\(misocket=/var/lib/mysql/mysql.sock \(mi\|\(miprint-defaults"
.B "$ mysqladmin \(mi\|\(misocket=/var/lib/mysql/mysql.sock variables"
.fi
.RE
.\"-----------------------------------------------
.TP
\(bu
The \*(Bi\& database is now ready for use.  Test
it like this from another user account:
.RS
.nf
.B "% mysql \(mi\|\(misocket=/var/lib/mysql/mysql.sock \(miu reader \(miD bibtex"
.B "\&\fCmysql> \&\fBselect count(*) from bibtab;\fP\fP"
\&\fC+----------+
| count(*) |
+----------+
|    11431 |
+----------+
1 row in set (0.00 sec)\fP
.fi
.RE
.\"-----------------------------------------------
.TP
\(bu
The last step is to ensure that the MySQL server
terminates safely when the system shuts down, and
starts when the system reboots.
.IP
The details depend on the operating system, but
the MySQL software installation may contain a
sample startup script in the file
\&\fC/usr/local/share/mysql/mysql.server\fP.  Your
own system may already have some scripts that can
be used after minor edits: look for files
\&\fC/etc/init.d/*sql*\fP,
\&\fC/etc/rc*.d/*sql*\fP, and
\&\fC/etc/mysql/*\fP.
.IP
The subdirectory \&\fCscripts\fP in the
.BR bibsql (1)
software distribution contains copies of such
scripts used at the author's site.
.IP
On a Red Hat GNU/Linux system, only two simple
changes to set location variables are needed:
.RS
.nf
.B "% diff /usr/local/share/mysql/mysql.server /etc/init.d/mysql"
\&\fC46,47c46,47
< basedir=
< datadir=
---
> basedir=/usr/local
> datadir=/var/lib/mysql\fP
.fi
.RE
.IP
Here is how to start, query, and stop the server:
.RS
.nf
.B "# /etc/init.d/mysql start"
\&\fCStarting MySQL.                    [  OK  ]\fP
.B ""
.B "# /etc/init.d/mysql status"
\&\fCMySQL running (8805)               [  OK  ]\fP
.B ""
.B "# /etc/init.d/mysql stop"
\&\fCShutting down MySQL..              [  OK  ]\fP
.fi
.RE
.IP
Symbolic links from the controlling script
\&\fC/etc/init.d/mysql\fP to two others make it
accessible to the Unix startup and shutdown
procedures:
.RS
.nf
.B "# cd /etc/init.d"
.B "# ln \(mis ../init.d/mysql ../rc3.d/S99mysql"
.B "# ln \(mis ../init.d/mysql ../rc0.d/K01mysql"
.fi
.RE
.IP
Should the \*(Bi\& database and the MySQL server
no longer be needed, then those two links must be
removed.  The database itself, and its filesystem
directory, can be removed with a single command in
a privileged client session:
.RS
.nf
.B "$ mysql \(mip \(mi\|\(misocket=/var/lib/mysql/mysql.sock \(miu root"
\&\fCEnter password: \fP\&\fB/s/e/c/r/e/t/\fP
\&\fC\&.\|.\|.\fP
\&\fCmysql> \fP\&\fBDROP DATABASE bibtex;\fP
\&\fCQuery OK, 3 rows affected (0.04 sec)\fP
.fi
.RE
.IP
There is no prompt to ask whether you really want
to remove the database.  To get it back, you
either need to recover it from filesystem backups,
or recreate it with the help of data from
.B bibtosql
and the two
.BR mysqladmin (1)
and
.BR mysql (1)
steps shown earlier in this section.
.PP
If the database file is large, searches may be
made faster by creating indexes of important parts
of the data:
.RS
.nf
\&\fCmysql> \fP\&\fBalter table bibtab add index (author(50),title(100),label(50));\fP
\&\fCmysql> \fP\&\fBalter table strtab add index (`key`(50), value(50));\fP
\&\fCmysql> \fP\&\fBalter table namtab add index (name(50));\fP
.fi
.RE
.PP
At the end of the next session, we discuss how to
automatically update the database by
regularly-scheduled jobs that find and insert new
\*(Bi\& data.
.\"=====================================================================
.SH "PostgreSQL SERVER SETUP"
Setting up the server side of a PostgreSQL
connection is a daunting process if you have never
done it.  These step-by-step notes may therefore
be helpful.  It is a good thing to log what you
are doing, either with the log facility provided
in an
.BR xterm (1)
window (usually on a menu bound to Ctl-Button-1),
or else with the
.BR script (1)
command.
.PP
We assume that the PostgreSQL software has been
installed in the tree \&\fC/usr/local/pgsql\fP,
and its databases are to be created under the
subdirectory \&\fCdata\fP.  For clarity, we use
three different prompt strings: \&\fC#\fP for the
\&\fCroot\fP user, \&\fC$\fP for the PostgreSQL
superuser, and \&\fC%\fP for an unprivileged
ordinary user.
.\"-----------------------------------------------
.TP \w'\(bu'u+2n
\(bu
If you have not yet done so, create a Unix account
for the PostgreSQL superuser:
.RS
.nf
.B "# adduser postgres"
.fi
.RE
.IP
This account has no special Unix privileges, and
exists only to restrict access to database files,
and to provide an account under which the
PostgreSQL server daemon runs.
.IP
In addition, it has no password set, so direct
login with this account is impossible.  Instead,
you must first become the \&\fCroot\fP user, and
then use the
.BR su (1)
or
.BR sudo (1)
commands to change to the \&\fCpostgres\fP account.
.\"-----------------------------------------------
.TP
\(bu
As the \&\fCroot\fP user, reset the ownership of two
subdirectories in the installation directory:
.RS
.nf
.B "# cd /usr/local/pgsql"
.B "# mkdir data"
.B "# mkdir logs"
.B "# chown \(miR postgres.postgres data logs"
.fi
.RE
.IP
For security and performance reasons, consider
making those two directories symbolic links to a
separate file tree, preferably on a local, rather
than network-mounted, disk.
.IP
All other files in the directory tree should be
owned by some other user, preferably an account
without login privileges, such as \&\fCbin\fP, so
that a compromised \&\fCpostgres\fP account cannot
be used to change those files.
.\"-----------------------------------------------
.TP
\(bu
Become the \&\fCpostgres\fP user, set the search
path to include PostgreSQL utilities, and
initialize the database directory tree with
template files:
.RS
.nf
.B "# su \(mi postgres"
.B "$ PATH=$PATH:/usr/local/pgsql/bin"
.B "$ export PATH"
.B "$ cd /usr/local/pgsql"
.B "$ initdb \(miD /usr/local/pgsql/data/bibtex"
.fi
.RE
.\"-----------------------------------------------
.TP
\(bu
Create two PostgreSQL user names (unrelated to
Unix accounts, and often called
.I roles
instead), one with permissions to create
databases, and another with permissions to just
read the bibliographic database:
.RS
.nf
.B "$ createuser bibtex"
\&\fCShall the new role be a superuser? (y/n) \&\fBn\fP
Shall the new role be allowed to create databases? (y/n) \&\fBy\fP
Shall the new role be allowed to create more new roles? (y/n) \&\fBn\fP\fP
.PP
.B "$ createuser reader"
\&\fCShall the new role be a superuser? (y/n) \&\fBn\fP
Shall the new role be allowed to create databases? (y/n) \&\fBn\fP
Shall the new role be allowed to create more new roles? (y/n) \&\fBn\fP\fP
.fi
.RE
.\"-----------------------------------------------
.TP
\(bu
Start the PostgreSQL server in its default mode
where connections are only accepted from clients
on the same host:
.RS
.nf
.B "$ pg_ctl \(miD /usr/local/pgsql/data/bibtex \(mil logs/server-`hostname`.log start"
.fi
.RE
.\"-----------------------------------------------
.TP
\(bu
Create the database \&\fCbibtex\fP:
.RS
.nf
.B "$ createdb /usr/local/pgsql/data/bibtex"
.fi
.RE
.\"-----------------------------------------------
.TP
\(bu
Populate the newly-created database with suitable
data produced by
.BR bibtosql ,
using the newly-created PostgreSQL \&\fCbibtex\fP
role:
.RS
.nf
.B "$ psql \(mid /usr/local/pgsql/data/bibtex \(miU bibtex \e"
.B "    < \&\fI/path/to/data/\fPtest-psql.sql \e"
.B "    > logs/bibtex-`hostname`.log 2>&1"
.fi
.RE
.IP
In the input redirection, supply the correct
directory path to the SQL input file.
.\"-----------------------------------------------
.TP
\(bu
Start the client program, check the status of the
two roles, assign access permissions to them, and
then check their permissions:
.RS
.nf
.B "$ psql \(mid /usr/local/pgsql/data/bibtex \(miU postgres"
\&\fCpostgres=# \&\fB\edu\fP
                               List of roles
 Role name | Superuser | Create role | Create DB | Connections | Member of
-----------+-----------+-------------+-----------+-------------+-----------
 reader    | no        | no          | no        | no limit    | {}
 bibtex    | no        | no          | yes       | no limit    | {}
 postgres  | yes       | yes         | yes       | no limit    | {}\fP
\&\fCpostgres=# \&\fBgrant all on bibtab, namtab, strtab to bibtex;\fP\fP
\&\fCpostgres=# \&\fBgrant select on bibtab, namtab, strtab to reader;\fP\fP
\&\fCpostgres=# \&\fB\ez\fP\fP
\&\fC  Access privileges for database "/usr/local/pgsql/data/bibtex"
 Schema | Name  | Type  |           Access privileges
--------+-------+-------+----------------------------------------
 public | bibtab | table | {bibtex=arwdxt/bibtex,reader=r/bibtex}
 public | namtab | table | {bibtex=arwdxt/bibtex,reader=r/bibtex}
 public | strtab | table | {bibtex=arwdxt/bibtex,reader=r/bibtex}\fP
.fi
.RE
.\"-----------------------------------------------
.TP
\(bu
As an ordinary user, start the SQL client program
on the current host with the \&\fCreader\fP
role, verify that it can see the newly-created
database and its newly-added contents, and try to
alter the database, an action that should be denied:
.RS
.nf
.B "% psql \(mid /usr/local/pgsql/data/bibtex \(miU reader"
\&\fC/usr/local/pgsql/data/bibtex=> \&\fB\el\fP
                 List of databases
            Name              |  Owner   | Encoding
------------------------------+----------+----------
 /usr/local/pgsql/data/bibtex | postgres | UTF8
 postgres                     | postgres | UTF8
 template0                    | postgres | UTF8
 template1                    | postgres | UTF8\fP
\&\fC/usr/local/pgsql/data/bibtex=> \&\fBselect count(*) from strtab;\fP\fP
\&\fC count
-------
   920\fP
\&\fC/usr/local/pgsql/data/bibtex=> \fP\&\fBinsert into bibtab (author, title) values('cat', 'dog');\fP
\&\fCERROR:  permission denied for relation bibtab\fP
\&\fC/usr/local/pgsql/data/bibtex=> \fP\&\fBinsert into strtab values('larry', 'curly', 'moe');\fP
\&\fCERROR:  permission denied for relation strtab\fP
\&\fC/usr/local/pgsql/data/bibtex=> \fP\&\fBinsert into namtab values ('Joe Blow', 0);\fP
\&\fCERROR:  permission denied for relation namtab\fP
.fi
.RE
.IP
At this point, users on the server host can run
the client program and successfully access the
database, and the \&\fCreader\fP role cannot
modify the database.
.\"-----------------------------------------------
.TP
\(bu
Enable network access to the server by editing the
PostgreSQL installation file
\&\fCdata/bibtex/pg_hba.conf\fP to specify the
Internet numeric addresses
.I "from which"
client connections are to be accepted.  For
example, we could add these lines at the end of
the file:
.RS
.nf
\&\fChost /usr/local/pgsql/data/bibtex bibtex 192.168.32.0/24 password\fP
\&\fChost /usr/local/pgsql/data/bibtex bibtex 192.168.96.0/26 password\fP
\&\fChost /usr/local/pgsql/data/bibtex reader 192.168.32.0/24 trust\fP
\&\fChost /usr/local/pgsql/data/bibtex reader 192.168.96.0/26 trust\fP
.fi
.RE
.IP
On the first two lines, the \&\fCpassword\fP
keyword says that user \&\fCbibtex\fP must supply
a password (set later) to access the database.  We
allow connections to the \&\fCbibtex\fP database
for two PostgreSQL roles from 256 addresses
beginning \&\fC192.168.32\fP, and 64 addresses
beginning \&\fC192.168.96\fP.  The trailing
\&\fC/24\fP means that the leading 24 bits of the
incoming address must match those of the pattern,
so the last 8 bits can represent any of 256 hosts.
A trailing \&\fC/32\fP restricts connections to a
single host.
.\"-----------------------------------------------
.TP
\(bu
Edit the file \&\fCdata/postgresql.conf\fP
to have a line like this:
.RS
.nf
.B "listen_addresses = '*'"
.fi
.RE
.IP
This tells the PostgreSQL server to listen on all
local network connections for incoming client
requests.
.IP
If you have multiple networks, one of which
reaches the outside world, and the others are
``inside the fence'', then you can replace the
asterisk by a comma-separated list of inside
addresses of the server, like this:
.RS
.nf
.B "listen_addresses = '192.168.96.10, 192.168.96.11, 192.168.96.12'"
.fi
.RE
.IP
The server can then respond to clients inside
your organization, but cannot offer database
access to clients elsewhere on the Internet.
.\"-----------------------------------------------
.TP
\(bu
As the \&\fCpostgres\fP user, restart the SQL
server so that it rereads the changed
configuration file that enables listening for a
network connection from a client.  Then check that
the PostgreSQL port 5432 is being listened to:
.RS
.nf
.B "$ pg_ctl \(miD /usr/local/pgsql/data/bibtex \(mio '\(mid 5' \(mil logs/server-`hostname`.log restart"
.B "$ netstat \(mian | grep 5432"
\&\fCtcp  0 0 0.0.0.0:5432        0.0.0.0:*           LISTEN
tcp  0 0 192.168.96.157:5432 192.168.96.19:36214 ESTABLISHED
tcp  0 0 :::5432             :::*                LISTEN
unix 2 [ ACC ]     STREAM   LISTENING     1119331 /tmp/.s.PGSQL.5432
unix 3 [ ]         STREAM   CONNECTED     1124578 /tmp/.s.PGSQL.5432\fP
.fi
.RE
.IP
The \&\fC\(mid 5\fP option requests maximal
debug information in the log file, which is a good
thing to do until you are sure the system is
operating properly.  The
.BR pg_ctl (1)
program passes that option to
.BR postmaster (1),
the PostgreSQL program that handles the startup of
an SQL server process to respond to the client
requests.  That way, a single server system can
provide simultaneous SQL access for multiple
databases.
.IP
The \&\fCtcp\fP lines verify that the network port
is being monitored, and the \&\fCunix\fP lines
show that the socket port for local connections is
also active.
.\"-----------------------------------------------
.TP
\(bu
On another host, as an ordinary user, check
whether the server is reachable from the client
(change the sample host name):
.RS
.nf
.B "% psql \(mid /usr/local/pgsql/data/bibtex \(miU reader \(mih server.example.com"
\&\fC/usr/local/pgsql/data/bibtex=> \&\fBselect count(*) from strtab;\fP
 count
-------
   920\fP
.fi
.RE
.IP
You can make the same check of the network
connection from the server host by changing the
hostname to \&\fClocalhost\fP, and in another
shell session on that host, requesting a network
status report:
.RS
.nf
.B "% psql \(mid /usr/local/pgsql/data/bibtex \(miU reader \(mih localhost"
.B "% netstat \(mian | grep 5432"
\&\fCtcp  0  0 127.0.0.1:5432  0.0.0.0:*        LISTEN
tcp  0  0 127.0.0.1:5432  127.0.0.1:33126  ESTABLISHED
tcp  0  0 127.0.0.1:33126 127.0.0.1:5432   ESTABLISHED
tcp  0  0 ::1:5432  :::*                   LISTEN
unix 2  [ ACC ]  STREAM  LISTENING  15814  /tmp/.s.PGSQL.5432\fP
.fi
.RE
.IP
The address \&\fC127.0.0.1\fP is that of the
.I loopback
interface, which provides a way for client and
server to be on the same system, while still using
normal TCP/IP network system calls, instead of
sockets, for communication.
.\"-----------------------------------------------
.TP
\(bu
Our PostgreSQL client/server setup is now
operational, but we need to tighten security
further.  We should restrict the \&\fCpostgres\fP
and \&\fCbibtex\fP roles so that they can be used
only on the server system, and so that at least
the first of them requires a database password.
.RS
.nf
.B "$ psql \(mid /usr/local/pgsql/data/bibtex \(miU postgres"
\&\fC/usr/local/pgsql/data/bibtex=# \&\fBalter user postgres password 's/e/c/r/e/t'\fP\fP
\&\fC/usr/local/pgsql/data/bibtex=# \&\fBalter user bibtex password 't/e/r/c/e/s'\fP\fP
.fi
.RE
.\"-----------------------------------------------
.TP
\(bu
In the \&\fC/etc\fP directory tree, install a
script that is to be executed when the server
machine starts up and shuts down.  The script
should start and stop the database.  The details
vary substantially across different flavors of
Unix, so we leave that job to you, the local
software installer.
.IP
PostgreSQL software distributions contain a few
sample scripts in the directory tree
\&\fCpostgresql-x.y.z/contrib/start-scripts\fP.
Your own system may already have some that can be
used after minor edits: look for files
\&\fC/etc/init.d/*sql*\fP,
\&\fC/etc/rc.d/*sql*\fP,
\&\fC/etc/pgsql/*\fP, and
\&\fC/etc/postgresql/*\fP.
.IP
.IP
The subdirectory \&\fCscripts\fP in the
.BR bibsql (1)
software distribution contains copies of such
scripts used at the author's site.
.IP
The \&\fCpostgres\fP login directory needs
a shell startup file that sets the \&\fCPATH\fP
variable to include the directory where PostgreSQL
programs are found:
.RS
.nf
.B "# ypmatch postgres passwd"
\&\fCpostgres:*:14255:14255:Database PostgreSQL:/var/pgsql:/bin/sh\fP
.B ""
.B "# cat >> /var/pgsql/.profile"
.B "PATH=$PATH:/usr/local/pgsql/bin"
.B "export PATH"
.B "^d"
.fi
.RE
.\"-----------------------------------------------
.TP
\(bu
If your \*(Bi\& files change, the database should
be updated.  It is then useful to set up a
.BR cron (8)
job to be run at suitable intervals on the
database server under the \&\fCpostgres\fP user
account.  The job reads a list of directories
containing \*(Bi\& files, uses
.BR find (1)
to identify the files that have changed since the
last update, and then runs
.B bibtosql
to convert them into an input stream for
.BR psql (1)
under the role \&\fCbibtex\fP.  Entries that are
unchanged from the last update are ignored because
of the \&\fCUNIQUE\fP attribute on the
\&\fCentry\fP column, but any that have been
changed or added are inserted into the database.
This is easier than it sounds:
.RS
.nf
.B "$ find `cat BIBTEXFILEPATHS` \(miname '*.bib' \e"
.B "        \(mia  \(mitype f \e"
.B "        \(mia  \(miperm \(mi444 \e"
.B "        \(mia \(minewer LAST_TIMESTAMP | \e"
.B "    xargs bibtosql \(mis psql | \e"
.B "    psql \(mid /usr/local/pgsql/data/bibtex \(miU bibtex"
.B "$ touch LAST_TIMESTAMP"
.fi
.RE
.IP
To avoid entering erroneously-formatted entries
into the database, use
.BR bibparse (1)
to validate each \*(Bi\& file before permitting it
to be seen by
.BR bibtosql .
.PP
If the database file is large, searches may be
made faster by creating indexes of important parts
of the data:
.RS
.nf
\&\fCpsql> \fP\&\fBcreate index bibidx on bibtab (author,title,label);\fP
\&\fCpsql> \fP\&\fBcreate index namidx on namtab (name);\fP
\&\fCpsql> \fP\&\fBcreate index stridx on strtab (key, value);\fP
.RE
.PP
.\"=====================================================================
.SH AUTHOR
.nf
Nelson H. F. Beebe
University of Utah
Department of Mathematics, 110 LCB
155 S 1400 E RM 233
Salt Lake City, UT 84112-0090
USA
Tel: +1 801 581 5254
FAX: +1 801 581 4148
Email: \&\fCbeebe@math.utah.edu\fP, \&\fCbeebe@acm.org\fP, \&\fCbeebe@computer.org\fP (Internet)
URL: \&\fChttp://www.math.utah.edu/~beebe\fP
.fi
.PP
The master archive for the
.BR bibsql (1)
and
.B bibtosql
software is at these equivalent locations:
.RS
.nf
\&\fCftp://ftp.math.utah.edu/pub/tex/bibsql\fP
.br
\&\fChttp://www.math.utah.edu/pub/tex/bibsql\fP
.fi
.RE
.\"=====================================================================
.SH "SEE ALSO"
.BR bib2xml (1),
.BR bib2xtx (1),
.BR bibcheck (1),
.BR bibclean (1),
.BR bibdestringify (1),
.BR bibdup (1),
.BR bibextract (1),
.BR bibindex (1),
.BR bibjoin (1),
.BR biblabel (1),
.BR biblex (1),
.BR biblook (1),
.BR biborder (1),
.BR bibparse (1),
.BR bibsearch (1),
.BR bibsort (1),
.BR bibsplit (1),
.BR bibsql (1),
.BR bibtex (1),
.BR bibunlex (1),
.BR cattobib (1),
.BR citefind (1),
.BR citesub (1),
.BR citetags (1),
.BR cron (8),
.BR date (1),
.BR find (1)
.BR latex (1),
.BR mysql (1),
.BR pg_ctl (1),
.BR postmaster (1),
.BR psql (1),
.BR ref2bib (1),
.BR scribe (1),
.BR script (1),
.BR sqlite3 (1),
.BR tex (1),
.BR xml2bib (1),
.BR xml2wordbib (1),
.BR xterm (1),
.BR xtx2bib (1).
.\"=====================================================================
.\" This is for GNU Emacs file-specific customization:
.\" Local Variables:
.\" fill-column: 50
.\" End:
